import _extends from "@babel/runtime/helpers/extends";
import _asyncToGenerator from "@babel/runtime/helpers/asyncToGenerator";
import _regeneratorRuntime from "@babel/runtime/regenerator";
/* eslint-disable no-param-reassign */
import { createElement } from 'react';
import { render, unmountComponentAtNode } from 'react-dom';
import { globalContext, Editor, engineConfig } from '@alilc/lowcode-editor-core';
import { Designer, LowCodePluginManager } from '@alilc/lowcode-designer';
import { Skeleton as InnerSkeleton, SettingsPrimaryPane, registerDefaults } from '@alilc/lowcode-editor-skeleton';
import Outline, { OutlineBackupPane, getTreeMaster } from '@alilc/lowcode-plugin-outline-pane';
import DesignerPlugin from '@alilc/lowcode-plugin-designer';
import { Hotkey, Project, Skeleton, Setters, Material, Event, Common } from '@alilc/lowcode-shell';
import { getLogger, isPlainObject } from '@alilc/lowcode-utils';
import './modules/live-editing';
import classes from './modules/classes';
import symbols from './modules/symbols';
import { shellModelFactory } from './modules/shell-model-factory';
export * from './modules/skeleton-types';
export * from './modules/designer-types';
export * from './modules/lowcode-types';
registerDefaults();
var editor = new Editor();
globalContext.register(editor, Editor);
globalContext.register(editor, 'editor');
var innerSkeleton = new InnerSkeleton(editor);
editor.set('skeleton', innerSkeleton);
var designer = new Designer({
  editor: editor,
  shellModelFactory: shellModelFactory
});
editor.set('designer', designer);
var innerProject = designer.project;
var hotkey = new Hotkey();
var project = new Project(innerProject);
var skeleton = new Skeleton(innerSkeleton);
var setters = new Setters();
var material = new Material(editor);
var config = engineConfig;
var event = new Event(editor, {
  prefix: 'common'
});
var logger = getLogger({
  level: 'warn',
  bizName: 'common'
});
var common = new Common(editor, innerSkeleton);
var pluginContextApiAssembler = {
  assembleApis: function assembleApis(context) {
    context.hotkey = hotkey;
    context.project = project;
    context.skeleton = skeleton;
    context.setters = setters;
    context.material = material;
    context.event = event;
    context.config = config;
    context.common = common;
  }
};
var plugins = new LowCodePluginManager(pluginContextApiAssembler).toProxy();
editor.set('plugins', plugins);
export { skeleton, plugins, project, setters, material, config, event, logger, hotkey, common,
// 兼容原 editor 的事件功能
event as editor };
// declare this is open-source version
export var isOpenSource = true;
export var __SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED = {
  symbols: symbols,
  classes: classes
};
engineConfig.set('isOpenSource', isOpenSource);

// 注册一批内置插件
(function () {
  var _registerPlugins = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee() {
    var componentMetaParser, setterRegistry, defaultPanelRegistry;
    return _regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            // 处理 editor.set('assets')，将组件元数据创建好
            componentMetaParser = function componentMetaParser(ctx) {
              return {
                init: function init() {
                  editor.onGot('assets', function (assets) {
                    var _assets$components = assets.components,
                      components = _assets$components === void 0 ? [] : _assets$components;
                    designer.buildComponentMetasMap(components);
                  });
                }
              };
            };
            componentMetaParser.pluginName = '___component_meta_parser___';
            _context.next = 4;
            return plugins.register(componentMetaParser);
          case 4:
            // 注册默认的 setters
            setterRegistry = function setterRegistry(ctx) {
              return {
                init: function init() {
                  var _require;
                  if (engineConfig.get('disableDefaultSetters')) return;
                  var builtinSetters = (_require = require('@alilc/lowcode-engine-ext')) === null || _require === void 0 ? void 0 : _require.setters;
                  if (builtinSetters) {
                    ctx.setters.registerSetter(builtinSetters);
                  }
                }
              };
            };
            setterRegistry.pluginName = '___setter_registry___';
            _context.next = 8;
            return plugins.register(setterRegistry);
          case 8:
            // 注册默认的面板
            defaultPanelRegistry = function defaultPanelRegistry(ctx) {
              return {
                init: function init() {
                  var _editor$getPreference;
                  skeleton.add({
                    area: 'mainArea',
                    name: 'designer',
                    type: 'Widget',
                    content: DesignerPlugin
                  });
                  if (!engineConfig.get('disableDefaultSettingPanel')) {
                    skeleton.add({
                      area: 'rightArea',
                      name: 'settingsPane',
                      type: 'Panel',
                      content: SettingsPrimaryPane,
                      props: {
                        ignoreRoot: true
                      }
                    });
                  }

                  // by default in float area;
                  var isInFloatArea = true;
                  var hasPreferenceForOutline = editor === null || editor === void 0 ? void 0 : (_editor$getPreference = editor.getPreference()) === null || _editor$getPreference === void 0 ? void 0 : _editor$getPreference.contains('outline-pane-pinned-status-isFloat', 'skeleton');
                  if (hasPreferenceForOutline) {
                    var _editor$getPreference2;
                    isInFloatArea = editor === null || editor === void 0 ? void 0 : (_editor$getPreference2 = editor.getPreference()) === null || _editor$getPreference2 === void 0 ? void 0 : _editor$getPreference2.get('outline-pane-pinned-status-isFloat', 'skeleton');
                  }
                  skeleton.add({
                    area: 'leftArea',
                    name: 'outlinePane',
                    type: 'PanelDock',
                    content: Outline,
                    panelProps: _extends({
                      area: isInFloatArea ? 'leftFloatArea' : 'leftFixedArea',
                      keepVisibleWhileDragging: true
                    }, engineConfig.get('defaultOutlinePaneProps'))
                  });
                  skeleton.add({
                    area: 'rightArea',
                    name: 'backupOutline',
                    type: 'Panel',
                    props: {
                      condition: function condition() {
                        return designer.dragon.dragging && !getTreeMaster(designer).hasVisibleTreeBoard();
                      }
                    },
                    content: OutlineBackupPane
                  });
                }
              };
            };
            defaultPanelRegistry.pluginName = '___default_panel___';
            _context.next = 12;
            return plugins.register(defaultPanelRegistry);
          case 12:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));
  function registerPlugins() {
    return _registerPlugins.apply(this, arguments);
  }
  return registerPlugins;
})()();

// container which will host LowCodeEngine DOM
var engineContainer;
// @ts-ignore webpack Define variable
export var version = VERSION_PLACEHOLDER;
engineConfig.set('ENGINE_VERSION', version);
export function init(_x, _x2, _x3) {
  return _init.apply(this, arguments);
}
function _init() {
  _init = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee2(container, options, pluginPreference) {
    var engineOptions, Workbench;
    return _regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            _context2.next = 2;
            return destroy();
          case 2:
            engineOptions = null;
            if (isPlainObject(container)) {
              engineOptions = container;
              engineContainer = document.createElement('div');
              engineContainer.id = 'engine';
              document.body.appendChild(engineContainer);
            } else {
              engineOptions = options;
              engineContainer = container;
              if (!container) {
                engineContainer = document.createElement('div');
                engineContainer.id = 'engine';
                document.body.appendChild(engineContainer);
              }
            }
            engineConfig.setEngineOptions(engineOptions);
            _context2.next = 7;
            return plugins.init(pluginPreference);
          case 7:
            Workbench = common.skeletonCabin.Workbench;
            render( /*#__PURE__*/createElement(Workbench, {
              skeleton: innerSkeleton,
              className: 'engine-main',
              topAreaItemClassName: 'engine-actionitem'
            }), engineContainer);
          case 9:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2);
  }));
  return _init.apply(this, arguments);
}
export function destroy() {
  return _destroy.apply(this, arguments);
}
function _destroy() {
  _destroy = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee3() {
    var documents;
    return _regeneratorRuntime.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            // remove all documents
            documents = project.documents;
            if (Array.isArray(documents) && documents.length > 0) {
              documents.forEach(function (doc) {
                return project.removeDocument(doc);
              });
            }

            // TODO: delete plugins except for core plugins

            // unmount DOM container, this will trigger React componentWillUnmount lifeCycle,
            // so necessary cleanups will be done.
            engineContainer && unmountComponentAtNode(engineContainer);
          case 3:
          case "end":
            return _context3.stop();
        }
      }
    }, _callee3);
  }));
  return _destroy.apply(this, arguments);
}